.device ATtiny13
.include "tn13def.inc"
.list
;--   
.def     w=r16        ;    
.def     setup=r17    ;  
.def     current=r18  ;   
.def     maximum=r19  ;  
.def     minimum=r20  ;  
.def     flags=r21    ;  (0-  , 1-   , 2 - slow/fast)
.def     setup_counter=r22
.def     setup_counter2=r23
.def     setup_counter3=r24
.def     output_counter=r25
.def     output_counter2=r26
.def     counter_sch=r27
;--------------------------
.equ     slow_setup=5     ;     
.equ     fast_setup=1     ;     
.equ     output_time=1    ;    
.equ     s_maximum=238    ;    
.equ     s_minimum=24     ;    
.equ     s_setup=128      ;   
;--  
; SPL -   
; ACSR -  
; DDRB -   
; PORTB -  
; PCMSK -     
; GIMSK -     
; GTCCR -  
; TCCR0B -  
; TIMSK0 -   
; PB0 -  , PB1 -  2, PB2-  1, PB3 - , PB4 -  
;--   
        .cseg
        .org 0
        rjmp Init  ;    
;--  
        reti       ; INT0
		rjmp Pch   ; Pin Change
		reti       ; Timer
		reti       ; EEPROM
		reti       ; comparator
		reti       ; timer compare match A
		reti       ; timer compare match B
		reti       ; watchdog
		reti       ; ADC
;--  
Init:   ldi w,RAMEND     ;   
        out SPL,w        ;     RAM
        sbi ACSR,7       ;  
        ldi w,0b00001001 ;      (1-, 0-)
        out DDRB,w
		ldi w,0b11100110 ;   
		out PORTB,w      ;     
;--    
        ldi maximum,s_maximum
		ldi minimum,s_minimum
		ldi setup,s_setup ;   
		ldi output_counter,255
	    ldi output_counter2,output_time
;--     PB0 (OC0A)
        ldi w,128        ;     - 1/256
		out OCR0A,w
        ldi w,0b10000011 ; fastPWM, set at TOP, clear at compare
		out TCCR0A,w
        ldi w,0b00000001 ; fastPWM, no_prescale
		out TCCR0B,w
;--    
        ldi w,0b00100000
		out GIMSK,w      ;    
;--     PB1, PB2
Start:  ldi w,0b00100000
        out GIFR,w       ;     
        ldi w,0b00000110
		out PCMSK,w      ;     PB2, PB1
        sei              ;   

;----------------------------------------------------------------------
Scan:   sbrc flags,0     ;    -  
        rcall Setup_update
        sbrc flags,1     ;    -    
        rcall Change_output
		rjmp Scan

;---   ---------------
Change_output:
        dec output_counter
        brne End_cho          ;      (Z=0) - 
		ldi output_counter,255
        dec output_counter2
        brne End_cho          ;      (Z=0) - 
        ;---  ,     -----
        in current,OCR0A      ;    
		cp current,setup      ; current - setup
        breq Equal            ;    
        brcc SetupIsLower;     - setup 
SetupIsHigher:
        inc current           ; 
		out OCR0A,current     ;   
		rjmp Exit
SetupIsLower:
		dec current
		out OCR0A,current     ;   
		rjmp Exit
Equal:  cbr flags,0b00000010  ;   ,  
        rjmp End_cho          ;  
Exit:   ldi output_counter,255
	    ldi output_counter2,output_time
End_cho:
	    ret

;---   -----------
Setup_update:
        dec  setup_counter
		brne End_chs   ;      (Z=1) -  
		ldi  setup_counter,255
		dec  setup_counter2
		brne End_chs   ;      (Z=1) -  
		ldi  setup_counter2,255
		dec  setup_counter3
		brne End_chs   ;      (Z=1) -  
Setup_change:
        sbis PINB,1    ;      - 
		rjmp Dec_setup
		sbis PINB,2    ;      - 
		rjmp Inc_setup
		cbr  flags,0b00000001 ;   
		rjmp End_chs
Dec_setup:
		dec setup
		cp  setup,minimum
		brlo SetMin       ;      -  
		rjmp ExitSetup
SetMin: mov setup,minimum ;     
        rjmp ExitSetup
Inc_Setup:
		inc setup
        cp maximum,setup
		brlo SetMax       ;      -  
		rjmp ExitSetup
SetMax: mov setup,maximum
ExitSetup:
        sbr flags,0b00000010   ;   
        ldi setup_counter, 255 ;    (    -   )
		ldi setup_counter2,255
		sbrc flags,2           ;    - fast 
        rjmp Fast_change
Slow_change:
		inc  counter_sch       ;    
        sbrc counter_sch,2     ;  2-   -    
        sbr  flags,0b00000100  ; fast change
        ldi  setup_counter3,slow_setup
		rjmp End_chs
Fast_change:
		ldi setup_counter3,fast_setup
End_chs:
        ret

;--    
Pch:       sbr  flags,0b00000001  ;    
           ldi  setup_counter,255
		   ldi  setup_counter2,1
		   ldi  setup_counter3,1
		   clr  counter_sch       ;   
		   cbr  flags,0b00000100  ; slow change
           reti                   ;  
